// qmidictlAbout.h
//
/****************************************************************************
   Copyright (C) 2010-2025, rncbc aka Rui Nuno Capela. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*****************************************************************************/

#ifndef __qmidictlAbout_h
#define __qmidictlAbout_h

#include "config.h"

#define QMIDICTL_TITLE      PROJECT_TITLE

#define QMIDICTL_SUBTITLE   PROJECT_DESCRIPTION
#define QMIDICTL_WEBSITE    PROJECT_HOMEPAGE_URL

#define QMIDICTL_COPYRIGHT  PROJECT_COPYRIGHT
#define QMIDICTL_DOMAIN     PROJECT_DOMAIN

#endif  // __qmidictlAbout_h

// end of qmidictlAbout.h

